@@ -1,4 +1,4 @@
-# Copyright 2009-2013 Yelp and Contributors
+# Copyright 2009-2016 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,40 +13,26 @@
 # limitations under the License.
 """Class to inherit your MapReduce jobs from. See :doc:`guides/writing-mrjobs`
 for more information."""
+
 # don't add imports here that aren't part of the standard Python library,
 # since MRJobs need to run in Amazon's generic EMR environment
-from __future__ import with_statement
-
-import codecs
 import inspect
 import itertools
+import json
 import logging
-from optparse import OptionGroup
 import sys
-
-try:
-    from cStringIO import StringIO
-    StringIO  # quiet "redefinition of unused ..." warning from pyflakes
-except ImportError:
-    from StringIO import StringIO
-
-try:
-    import simplejson as json
-    json  # silence, pyflakes!
-except ImportError:
-    import json
+from optparse import OptionGroup
 
 # don't use relative imports, to allow this script to be invoked as __main__
 from mrjob.conf import combine_dicts
-
-from mrjob.parse import parse_mr_job_stderr
 from mrjob.protocol import JSONProtocol
 from mrjob.protocol import RawValueProtocol
 from mrjob.launch import MRJobLauncher
 from mrjob.launch import _READ_ARGS_FROM_SYS_ARGV
-from mrjob.step import JarStep
 from mrjob.step import MRStep
 from mrjob.step import _JOB_STEP_FUNC_PARAMS
+from mrjob.py2 import integer_types
+from mrjob.py2 import string_types
 from mrjob.util import read_input
 
 
@@ -68,6 +54,20 @@ _SORT_VALUES_PARTITIONER = \
     'org.apache.hadoop.mapred.lib.KeyFieldBasedPartitioner'
 
 
+def _im_func(f):
+    """Wrapper to get at the underlying function belonging to a method.
+
+    Python 2 is slightly different because classes have "unbound methods"
+    which wrap the underlying function, whereas on Python 3 they're just
+    functions. (Methods work the same way on both versions.)
+    """
+    # "im_func" is the old Python 2 name for __func__
+    if hasattr(f, '__func__'):
+        return f.__func__
+    else:
+        return f
+
+
 class UsageError(Exception):
     pass
 
@@ -329,95 +329,35 @@ class MRJob(MRJobLauncher):
 
         :return: a list of steps constructed with :py:meth:`mr`
         """
-        # Use mapper(), reducer() etc. only if they've been re-defined
-        kwargs = dict((func_name, getattr(self, func_name))
-                      for func_name in _JOB_STEP_FUNC_PARAMS
-                      if (getattr(self, func_name).im_func is not
-                          getattr(MRJob, func_name).im_func))
+        # only include methods that have been redefined
+        kwargs = dict(
+            (func_name, getattr(self, func_name))
+            for func_name in _JOB_STEP_FUNC_PARAMS
+            if (_im_func(getattr(self, func_name)) is not
+                _im_func(getattr(MRJob, func_name))))
 
         # MRStep takes commands as strings, but the user defines them in the
         # class as functions that return strings, so call the functions.
         updates = {}
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             if k.endswith('_cmd'):
                 updates[k] = v()
 
         kwargs.update(updates)
 
-        return [self.mr(**kwargs)]
+        return [MRStep(**kwargs)]
 
     @classmethod
-    def mr(cls, *args, **kwargs):
-        """Define a Python step (mapper, reducer, and/or any combination of
-        mapper_init, reducer_final, etc.) for your job.
-
-        Used by :py:meth:`steps`. (Don't re-define this, just call it!)
-        See :ref:`writing-multi-step-jobs` for sample usage.
-
-        Accepts the following keyword arguments.
-
-        :param mapper: function with same function signature as
-                       :py:meth:`mapper`, or ``None`` for an identity mapper.
-        :param reducer: function with same function signature as
-                        :py:meth:`reducer`, or ``None`` for no reducer.
-        :param combiner: function with same function signature as
-                         :py:meth:`combiner`, or ``None`` for no combiner.
-        :param mapper_init: function with same function signature as
-                            :py:meth:`mapper_init`, or ``None`` for no initial
-                            mapper action.
-        :param mapper_final: function with same function signature as
-                             :py:meth:`mapper_final`, or ``None`` for no final
-                             mapper action.
-        :param reducer_init: function with same function signature as
-                             :py:meth:`reducer_init`, or ``None`` for no
-                             initial reducer action.
-        :param reducer_final: function with same function signature as
-                              :py:meth:`reducer_final`, or ``None`` for no
-                              final reducer action.
-        :param combiner_init: function with same function signature as
-                              :py:meth:`combiner_init`, or ``None`` for no
-                              initial combiner action.
-        :param combiner_final: function with same function signature as
-                               :py:meth:`combiner_final`, or ``None`` for no
-                               final combiner action.
-        :param jobconf: dictionary with custom jobconf arguments to pass to
-                        hadoop.
-
-        This is just a wrapper for :py:class:`~mrjob.step.MRStep`, plus
-        a little logic to support deprecated use of positional arguments.
+    def mr(cls, **kwargs):
+        """A deprecated wrapper for :py:class:`~mrjob.step.MRStep`.
         """
-        if args:
-            log.warning('Using positional arguments to MRJob.mr() is'
-                        ' deprecated and will be removed in v0.5.0')
-
-        if len(args) > 0:
-            kwargs['mapper'] = args[0]
-
-        if len(args) > 1:
-            kwargs['reducer'] = args[1]
-
-        if len(args) > 2:
-            raise ValueError('mr() can take at most two positional arguments.')
+        log.warning('mr() is deprecated and will be removed in v0.6.0.'
+                    ' Use mrjob.step.MRStep directly instead.')
 
         return MRStep(**kwargs)
 
-    @classmethod
-    def jar(cls, *args, **kwargs):
-        """Alias for :py:class:`~mrjob.step.JarStep`.
-
-        .. deprecated:: 0.4.2
-        """
-        log.warning('MRJob.jar() is deprecated and will be removed in'
-                    ' v0.5.0. Use mrjob.step.JarStep directly.')
-        return JarStep(*args, **kwargs)
-
     def increment_counter(self, group, counter, amount=1):
-        """Increment a counter in Hadoop streaming by printing to stderr. If
-        the type of either **group** or **counter** is ``unicode``, then the
-        counter will be written as unicode. Otherwise, the counter will be
-        written as ASCII. Although writing non-ASCII will succeed, the
-        resulting counter names may not be displayed correctly at the end of
-        the job.
+        """Increment a counter in Hadoop streaming by printing to stderr.
 
         :type group: str
         :param group: counter group
@@ -430,28 +370,29 @@ class MRJob(MRJobLauncher):
         with semicolons (commas confuse Hadoop streaming).
         """
         # don't allow people to pass in floats
-        if not isinstance(amount, (int, long)):
+        if not isinstance(amount, integer_types):
             raise TypeError('amount must be an integer, not %r' % (amount,))
 
+        # cast non-strings to strings (if people pass in exceptions, etc)
+        if not isinstance(group, string_types):
+            group = str(group)
+        if not isinstance(counter, string_types):
+            counter = str(counter)
+
         # Extra commas screw up hadoop and there's no way to escape them. So
         # replace them with the next best thing: semicolons!
         #
-        # cast to str() because sometimes people pass in exceptions or whatever
-        #
         # The relevant Hadoop code is incrCounter(), here:
-        # http://svn.apache.org/viewvc/hadoop/mapreduce/trunk/src/contrib/streaming/src/java/org/apache/hadoop/streaming/PipeMapRed.java?view=markup
-        if isinstance(group, unicode) or isinstance(counter, unicode):
-            group = unicode(group).replace(',', ';')
-            counter = unicode(counter).replace(',', ';')
-            stderr = codecs.getwriter('utf-8')(self.stderr)
-        else:
-            group = str(group).replace(',', ';')
-            counter = str(counter).replace(',', ';')
-            stderr = self.stderr
+        # http://svn.apache.org/viewvc/hadoop/mapreduce/trunk/src/contrib/streaming/src/java/org/apache/hadoop/streaming/PipeMapRed.java?view=markup  # noqa
+        group = group.replace(',', ';')
+        counter = counter.replace(',', ';')
 
-        stderr.write(
-            u'reporter:counter:%s,%s,%d\n' % (group, counter, amount))
-        stderr.flush()
+        line = 'reporter:counter:%s,%s,%d\n' % (group, counter, amount)
+        if not isinstance(line, bytes):
+            line = line.encode('utf_8')
+
+        self.stderr.write(line)
+        self.stderr.flush()
 
     def set_status(self, msg):
         """Set the job status in hadoop streaming by printing to stderr.
@@ -459,18 +400,13 @@ class MRJob(MRJobLauncher):
         This is also a good way of doing a keepalive for a job that goes a
         long time between outputs; Hadoop streaming usually times out jobs
         that give no output for longer than 10 minutes.
-
-        If the type of **msg** is ``unicode``, then the message will be written
-        as unicode. Otherwise, it will be written as ASCII.
         """
-        if isinstance(msg, unicode):
-            status = u'reporter:status:%s\n' % (msg,)
-            stderr = codecs.getwriter('utf-8')(self.stderr)
-        else:
-            status = 'reporter:status:%s\n' % (msg,)
-            stderr = self.stderr
-        stderr.write(status)
-        stderr.flush()
+        line = 'reporter:status:%s\n' % (msg,)
+        if not isinstance(line, bytes):
+            line = line.encode('utf_8')
+
+        self.stderr.write(line)
+        self.stderr.flush()
 
     ### Running the job ###
 
@@ -540,10 +476,10 @@ class MRJob(MRJobLauncher):
         :type step_num: int
         :param step_num: which step to run (0-indexed)
 
-        If we encounter a line that can't be decoded by our input protocol,
-        or a tuple that can't be encoded by our output protocol, we'll
-        increment a counter rather than raising an exception. If
-        --strict-protocols is set, then an exception is raised
+        If --no-strict-protocols is set, and we encounter a line that can't
+        be decoded by our input protocol or a tuple that can't be encoded
+        by our output protocol, we'll increment a counter rather than
+        raising an exception. This will be going away in v0.6.0.
 
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
@@ -608,7 +544,7 @@ class MRJob(MRJobLauncher):
         # be careful to use generators for everything, to allow for
         # very large groupings of values
         for key, kv_pairs in itertools.groupby(read_lines(),
-                                               key=lambda(k, v): k):
+                                               key=lambda k_v: k_v[0]):
             values = (v for k, v in kv_pairs)
             for out_key, out_value in reducer(key, values) or ():
                 write_line(out_key, out_value)
@@ -653,7 +589,7 @@ class MRJob(MRJobLauncher):
         # be careful to use generators for everything, to allow for
         # very large groupings of values
         for key, kv_pairs in itertools.groupby(read_lines(),
-                                               key=lambda(k, v): k):
+                                               key=lambda k_v1: k_v1[0]):
             values = (v for k, v in kv_pairs)
             for out_key, out_value in combiner(key, values) or ():
                 write_line(out_key, out_value)
@@ -670,11 +606,14 @@ class MRJob(MRJobLauncher):
 
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
-
-        We currently output something like ``MR M R``, but expect this to
-        change!
         """
-        print >> self.stdout, json.dumps(self._steps_desc())
+        # json only uses strings, but self.stdout only accepts bytes
+        steps_json = json.dumps(self._steps_desc())
+        if not isinstance(steps_json, bytes):
+            steps_json = steps_json.encode('utf_8')
+
+        self.stdout.write(steps_json)
+        self.stdout.write(b'\n')
 
     def _steps_desc(self):
         step_descs = []
@@ -685,8 +624,12 @@ class MRJob(MRJobLauncher):
     @classmethod
     def mr_job_script(cls):
         """Path of this script. This returns the file containing
-        this class."""
-        return inspect.getsourcefile(cls)
+        this class, or ``None`` if there isn't any (e.g. it was
+        defined from the command line interface.)"""
+        try:
+            return inspect.getsourcefile(cls)
+        except TypeError:
+            return None
 
     ### Other useful utilities ###
 
@@ -726,10 +669,12 @@ class MRJob(MRJobLauncher):
         def read_lines():
             for line in self._read_input():
                 try:
-                    key, value = read(line.rstrip('\r\n'))
+                    key, value = read(line.rstrip(b'\r\n'))
                     yield key, value
-                except Exception, e:
-                    if self.options.strict_protocols:
+                except Exception as e:
+                    # the strict_protocols option has to default to None
+                    # because it's used by runners, so treat None as true
+                    if self.options.strict_protocols is not False:
                         raise
                     else:
                         self.increment_counter(
@@ -737,9 +682,11 @@ class MRJob(MRJobLauncher):
 
         def write_line(key, value):
             try:
-                print >> self.stdout, write(key, value)
-            except Exception, e:
-                if self.options.strict_protocols:
+                self.stdout.write(write(key, value))
+                self.stdout.write(b'\n')
+            except Exception as e:
+                # None counts as true, see above
+                if self.options.strict_protocols is not False:
                     raise
                 else:
                     self.increment_counter(
@@ -804,18 +751,8 @@ class MRJob(MRJobLauncher):
             step_key = self._step_key(step_num, step_type)
 
             if step_key not in step_map:
-                # It's unlikely that we will encounter this logic in real life,
-                # but if asked what the protocol of a non-script step is, we
-                # should just say RawValueProtocol because we have no idea what
-                # the jars or commands are doing with our precious data.
-                # If --strict-protocols, though, we won't stand for these
-                # shenanigans!
-                if self.options.strict_protocols:
-                    raise ValueError(
-                        "Can't pick a protocol for a non-script step")
-                else:
-                    p = RawValueProtocol()
-                    return p, p
+                raise ValueError(
+                    "Can't pick a protocol for a non-script step")
 
             real_num = step_map[step_key]
             if real_num == (len(step_map) - 1):
@@ -890,8 +827,8 @@ class MRJob(MRJobLauncher):
     def all_option_groups(self):
         return super(MRJob, self).all_option_groups() + (self.mux_opt_group,)
 
-    def is_mapper_or_reducer(self):
-        """True if this is a mapper/reducer.
+    def is_task(self):
+        """True if this is a mapper, combiner, or reducer.
 
         This is mostly useful inside :py:meth:`load_options`, to disable
         loading options when we aren't running inside Hadoop Streaming.
@@ -923,8 +860,8 @@ class MRJob(MRJobLauncher):
         :py:attr:`INPUT_PROTOCOL`.
         """
         if not isinstance(self.INPUT_PROTOCOL, type):
-            log.warn('INPUT_PROTOCOL should be a class, not %s' %
-                     self.INPUT_PROTOCOL)
+            log.warning('INPUT_PROTOCOL should be a class, not %s' %
+                        self.INPUT_PROTOCOL)
         return self.INPUT_PROTOCOL()
 
     def internal_protocol(self):
@@ -933,8 +870,8 @@ class MRJob(MRJobLauncher):
         :py:attr:`INTERNAL_PROTOCOL`.
         """
         if not isinstance(self.INTERNAL_PROTOCOL, type):
-            log.warn('INTERNAL_PROTOCOL should be a class, not %s' %
-                     self.INTERNAL_PROTOCOL)
+            log.warning('INTERNAL_PROTOCOL should be a class, not %s' %
+                        self.INTERNAL_PROTOCOL)
         return self.INTERNAL_PROTOCOL()
 
     def output_protocol(self):
@@ -943,8 +880,8 @@ class MRJob(MRJobLauncher):
         :py:attr:`OUTPUT_PROTOCOL`.
         """
         if not isinstance(self.OUTPUT_PROTOCOL, type):
-            log.warn('OUTPUT_PROTOCOL should be a class, not %s' %
-                     self.OUTPUT_PROTOCOL)
+            log.warning('OUTPUT_PROTOCOL should be a class, not %s' %
+                        self.OUTPUT_PROTOCOL)
         return self.OUTPUT_PROTOCOL()
 
     #: Protocol for reading input to the first mapper in your job.
@@ -1074,7 +1011,7 @@ class MRJob(MRJobLauncher):
     JOBCONF = {}
 
     def jobconf(self):
-        """``-jobconf`` args to pass to hadoop streaming. This should be a map
+        """``-D`` args to pass to hadoop streaming. This should be a map
         from property name to value.
 
         By default, this combines :option:`jobconf` options from the command
@@ -1124,7 +1061,7 @@ class MRJob(MRJobLauncher):
                 # e.g. 1.0
                 return '%.1f' % v_float
             else:
-                # e.g. 0.18 or 0.20
+                # e.g. 0.20
                 return '%.2f' % v_float
 
         for key in unfiltered_jobconf:
@@ -1138,11 +1075,12 @@ class MRJob(MRJobLauncher):
                 else:
                     filtered_val = 'false'
 
+            # TODO: why would a jobconf variable be named 'hadoop_version'?
             # hadoop_version should be a string
             elif (key == 'hadoop_version' and
                     isinstance(unfiltered_val, float)):
-                log.warn('hadoop_version should be a string, not %s' %
-                         unfiltered_val)
+                log.warning('hadoop_version should be a string, not %s' %
+                            unfiltered_val)
                 filtered_val = format_hadoop_version(unfiltered_val)
             filtered_jobconf[key] = filtered_val
 
@@ -1176,53 +1114,6 @@ class MRJob(MRJobLauncher):
     #: .. versionadded:: 0.4.1
     SORT_VALUES = None
 
-    ### Testing ###
-
-    def parse_counters(self, counters=None):
-        """Convenience method for reading counters. This only works
-        in sandbox mode. This does not clear ``self.stderr``.
-
-        :return: a map from counter group to counter name to amount.
-
-        To read everything from ``self.stderr`` (including status messages)
-        use :py:meth:`mrjob.parse.parse_mr_job_stderr`.
-
-        When writing unit tests, you may find :py:meth:`MRJobRunner.counters()
-        <mrjob.runner.MRJobRunner.counters()>` more useful.
-        """
-        if self.stderr == sys.stderr:
-            raise AssertionError('You must call sandbox() first;'
-                                 ' parse_counters() is for testing only.')
-
-        stderr_results = parse_mr_job_stderr(self.stderr.getvalue(), counters)
-        return stderr_results['counters']
-
-    def parse_output(self, protocol=None):
-        """Convenience method for parsing output from any mapper or reducer,
-        all at once.
-
-        This helps you test individual mappers and reducers by calling
-        run_mapper() or run_reducer(). For example::
-
-            mr_job.sandbox(stdin=your_input)
-            mr_job.run_mapper(step_num=0)
-            output = mrjob.parse_output()
-
-        :type protocol: protocol
-        :param protocol: A protocol instance to use (e.g. JSONProtocol()),
-
-        This only works in sandbox mode. This does not clear ``self.stdout``.
-        """
-        if self.stdout == sys.stdout:
-            raise AssertionError('You must call sandbox() first;'
-                                 ' parse_output() is for testing only.')
-
-        if protocol is None:
-            protocol = JSONProtocol()
-
-        lines = StringIO(self.stdout.getvalue())
-        return [protocol.read(line) for line in lines]
-
 
 if __name__ == '__main__':
     MRJob.run()
