@@ -1,4 +1,6 @@
 # Copyright 2009-2012 Yelp and Contributors
+# Copyright 2013 David Marin
+# Copyright 2014-2016 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,35 +13,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import with_statement
-
+import codecs
 import logging
+import os
+import sys
+import time
+from io import BytesIO
 from optparse import Option
 from optparse import OptionError
 from optparse import OptionGroup
 from optparse import OptionParser
-import os
-import sys
-import time
-
-try:
-    from cStringIO import StringIO
-    StringIO  # quiet "redefinition of unused ..." warning from pyflakes
-except ImportError:
-    from StringIO import StringIO
 
 from mrjob.conf import combine_dicts
-from mrjob.options import add_basic_opts
-from mrjob.options import add_emr_opts
-from mrjob.options import add_hadoop_opts
-from mrjob.options import add_hadoop_emr_opts
-from mrjob.options import add_hadoop_shared_opts
-from mrjob.options import add_protocol_opts
-from mrjob.options import add_runner_opts
-from mrjob.options import print_help_for_groups
-from mrjob.parse import parse_key_value_list
-from mrjob.parse import parse_port_range_list
-from mrjob.runner import CLEANUP_CHOICES
+from mrjob.options import _add_basic_opts
+from mrjob.options import _add_emr_opts
+from mrjob.options import _add_hadoop_emr_opts
+from mrjob.options import _add_hadoop_opts
+from mrjob.options import _add_local_opts
+from mrjob.options import _add_protocol_opts
+from mrjob.options import _add_runner_opts
+from mrjob.options import _alphabetize_options
+from mrjob.options import _fix_custom_options
+from mrjob.options import _print_help_for_groups
+from mrjob.py2 import PY2
+from mrjob.step import StepFailedException
 from mrjob.util import log_to_null
 from mrjob.util import log_to_stream
 from mrjob.util import parse_and_save_options
@@ -51,6 +48,11 @@ log = logging.getLogger(__name__)
 # sentinel value; used when running MRJob as a script
 _READ_ARGS_FROM_SYS_ARGV = '_READ_ARGS_FROM_SYS_ARGV'
 
+# options only used to modify other options; don't pass to runners
+_FAKE_OPTIONS = set([
+    'no_emr_api_params',
+])
+
 
 class MRJobLauncher(object):
     """Handle running a MapReduce job on an executable from the command line.
@@ -92,6 +94,9 @@ class MRJobLauncher(object):
                                           add_help_option=False)
         self.configure_options()
 
+        for opt_group in self.all_option_groups():
+            _alphabetize_options(opt_group)
+
         # don't pass None to parse_args unless we're actually running
         # the MRJob script
         if args is _READ_ARGS_FROM_SYS_ARGV:
@@ -112,9 +117,16 @@ class MRJobLauncher(object):
 
         # Make it possible to redirect stdin, stdout, and stderr, for testing
         # See sandbox(), below.
-        self.stdin = sys.stdin
-        self.stdout = sys.stdout
-        self.stderr = sys.stderr
+        #
+        # These should always read/write bytes, not unicode
+        if PY2:
+            self.stdin = sys.stdin
+            self.stdout = sys.stdout
+            self.stderr = sys.stderr
+        else:
+            self.stdin = sys.stdin.buffer
+            self.stdout = sys.stdout.buffer
+            self.stderr = sys.stderr.buffer
 
     @classmethod
     def _usage(cls):
@@ -151,16 +163,13 @@ class MRJobLauncher(object):
 
         :rtype: :py:class:`mrjob.runner.MRJobRunner`
         """
-        # have to import here so that we can still run the MRJob
-        # without importing boto
-        from mrjob.emr import EMRJobRunner
-        from mrjob.hadoop import HadoopJobRunner
-        from mrjob.local import LocalMRJobRunner
-
         if self.options.runner == 'emr':
+            # avoid requiring dependencies (such as boto) for other runners
+            from mrjob.emr import EMRJobRunner
             return EMRJobRunner(**self.emr_job_runner_kwargs())
 
         elif self.options.runner == 'hadoop':
+            from mrjob.hadoop import HadoopJobRunner
             return HadoopJobRunner(**self.hadoop_job_runner_kwargs())
 
         elif self.options.runner == 'inline':
@@ -169,6 +178,7 @@ class MRJobLauncher(object):
 
         else:
             # run locally by default
+            from mrjob.local import LocalMRJobRunner
             return LocalMRJobRunner(**self.local_job_runner_kwargs())
 
     @classmethod
@@ -186,8 +196,10 @@ class MRJobLauncher(object):
         """
         if quiet:
             log_to_null(name='mrjob')
+            log_to_null(name='__main__')
         else:
             log_to_stream(name='mrjob', debug=verbose, stream=stream)
+            log_to_stream(name='__main__', debug=verbose, stream=stream)
 
         log_to_null(name='boto')
 
@@ -199,12 +211,22 @@ class MRJobLauncher(object):
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
         """
+        # self.stderr is strictly binary, need to wrap it so it's possible
+        # to log to it in Python 3
+        log_stream = codecs.getwriter('utf_8')(self.stderr)
+
         self.set_up_logging(quiet=self.options.quiet,
                             verbose=self.options.verbose,
-                            stream=self.stderr)
+                            stream=log_stream)
 
         with self.make_runner() as runner:
-            runner.run()
+            try:
+                runner.run()
+            except StepFailedException as e:
+                # no need for a runner stacktrace if step failed; runners will
+                # log more useful information anyway
+                log.error(str(e))
+                sys.exit(1)
 
             if not self.options.no_output:
                 for line in runner.stream_output():
@@ -240,6 +262,11 @@ class MRJobLauncher(object):
             default=False,
             help='show Hadoop-related options')
 
+        self.option_parser.add_option(
+            '--help-local', dest='help_local', action='store_true',
+            default=False,
+            help='show local/inline runner-related options')
+
         self.option_parser.add_option(
             '--help-runner', dest='help_runner', action='store_true',
             default=False, help='show runner-related options')
@@ -249,24 +276,23 @@ class MRJobLauncher(object):
             self.option_parser, 'Protocols')
         self.option_parser.add_option_group(self.proto_opt_group)
 
-        self._passthrough_options.extend(
-            add_protocol_opts(self.proto_opt_group))
+        _add_protocol_opts(self.proto_opt_group)
 
         # options for running the entire job
         self.runner_opt_group = OptionGroup(
             self.option_parser, 'Running the entire job')
         self.option_parser.add_option_group(self.runner_opt_group)
 
-        add_runner_opts(self.runner_opt_group, self._DEFAULT_RUNNER)
-        add_basic_opts(self.runner_opt_group)
+        _add_runner_opts(self.runner_opt_group, self._DEFAULT_RUNNER)
+        _add_basic_opts(self.runner_opt_group)
 
-        self.hadoop_opts_opt_group = OptionGroup(
+        # options for inline/local runners
+        self.local_opt_group = OptionGroup(
             self.option_parser,
-            'Configuring or emulating Hadoop (these apply when you set -r'
-            ' hadoop, -r emr, or -r local)')
-        self.option_parser.add_option_group(self.hadoop_opts_opt_group)
+            'Running locally (these apply when you set -r inline or -r local)')
+        self.option_parser.add_option_group(self.local_opt_group)
 
-        add_hadoop_shared_opts(self.hadoop_opts_opt_group)
+        _add_local_opts(self.local_opt_group)
 
         # options common to Hadoop and EMR
         self.hadoop_emr_opt_group = OptionGroup(
@@ -275,7 +301,7 @@ class MRJobLauncher(object):
             ' -r emr)')
         self.option_parser.add_option_group(self.hadoop_emr_opt_group)
 
-        add_hadoop_emr_opts(self.hadoop_emr_opt_group)
+        _add_hadoop_emr_opts(self.hadoop_emr_opt_group)
 
         # options for running the job on Hadoop
         self.hadoop_opt_group = OptionGroup(
@@ -283,30 +309,35 @@ class MRJobLauncher(object):
             'Running on Hadoop (these apply when you set -r hadoop)')
         self.option_parser.add_option_group(self.hadoop_opt_group)
 
-        add_hadoop_opts(self.hadoop_opt_group)
+        _add_hadoop_opts(self.hadoop_opt_group)
 
         # options for running the job on EMR
         self.emr_opt_group = OptionGroup(
             self.option_parser,
-            'Running on Amazon Elastic MapReduce (these apply when you set -r'
-            ' emr)')
+            'Running on EMR (these apply when you set -r emr)')
         self.option_parser.add_option_group(self.emr_opt_group)
 
-        add_emr_opts(self.emr_opt_group)
+        _add_emr_opts(self.emr_opt_group)
 
     def all_option_groups(self):
         return (self.option_parser, self.proto_opt_group,
                 self.runner_opt_group, self.hadoop_emr_opt_group,
-                self.emr_opt_group, self.hadoop_opts_opt_group)
+                self.emr_opt_group, self.local_opt_group)
 
-    def is_mapper_or_reducer(self):
-        """True if this is a mapper/reducer.
+    def is_task(self):
+        """True if this is a mapper, combiner, or reducer.
 
         This is mostly useful inside :py:meth:`load_options`, to disable
         loading options when we aren't running inside Hadoop Streaming.
         """
         return False
 
+    def is_mapper_or_reducer(self):
+        """The old name for :py:meth:`is_task`. Going away in v0.6.0"""
+        log.warning('is_mapper_or_reducer() has been renamed to is_task().'
+                    ' This alias will be removed in v0.6.0')
+        return self.is_task()
+
     def add_passthrough_option(self, *args, **kwargs):
         """Function to create options which both the job runner
         and the job itself respect (we use this for protocols, for example).
@@ -357,7 +388,7 @@ class MRJobLauncher(object):
             raise OptionError(
                 'passthrough file options must take strings' % pass_opt.type)
 
-        if not pass_opt.action in ('store', 'append'):
+        if pass_opt.action not in ('store', 'append'):
             raise OptionError("passthrough file options must use the options"
                               " 'store' or 'append'")
 
@@ -368,12 +399,12 @@ class MRJobLauncher(object):
         uses all args as input files. This method determines the behavior:
         MRJobLauncher takes off the first arg as the script path.
         """
-        if self._script_path:
-            self._script_path = os.path.abspath(args[0])
-        elif len(args) < 1:
-            self.option_parser.error('Must supply script path')
-
-        self.args = args[1:]
+        if not self._script_path:
+            if len(args) < 1:
+                self.option_parser.error('Must supply script path')
+            else:
+                self._script_path = os.path.abspath(args[0])
+                self.args = args[1:]
 
     def _help_main(self):
         self.option_parser.option_groups = []
@@ -381,7 +412,8 @@ class MRJobLauncher(object):
         sys.exit(0)
 
     def load_options(self, args):
-        """Load command-line options into ``self.options``.
+        """Load command-line options into ``self.options``,
+        ``self._script_path``, and ``self.args``.
 
         Called from :py:meth:`__init__()` after :py:meth:`configure_options`.
 
@@ -403,63 +435,26 @@ class MRJobLauncher(object):
             self._help_main()
 
         if self.options.help_emr:
-            print_help_for_groups(self.hadoop_emr_opt_group,
-                                  self.emr_opt_group)
+            _print_help_for_groups(self.hadoop_emr_opt_group,
+                                   self.emr_opt_group)
             sys.exit(0)
 
         if self.options.help_hadoop:
-            print_help_for_groups(self.hadoop_emr_opt_group,
-                                  self.hadoop_opts_opt_group)
+            _print_help_for_groups(self.hadoop_emr_opt_group,
+                                   self.hadoop_opt_group)
+            sys.exit(0)
+
+        if self.options.help_local:
+            _print_help_for_groups(self.local_opt_group)
             sys.exit(0)
 
         if self.options.help_runner:
-            print_help_for_groups(self.runner_opt_group)
+            _print_help_for_groups(self.runner_opt_group)
             sys.exit(0)
 
         self._process_args(args)
 
-        # parse custom options here to avoid setting a custom Option subclass
-        # and confusing users
-
-        if self.options.ssh_bind_ports:
-            try:
-                ports = parse_port_range_list(self.options.ssh_bind_ports)
-            except ValueError, e:
-                self.option_parser.error('invalid port range list "%s": \n%s' %
-                                         (self.options.ssh_bind_ports,
-                                          e.args[0]))
-            self.options.ssh_bind_ports = ports
-
-        cmdenv_err = 'cmdenv argument "%s" is not of the form KEY=VALUE'
-        self.options.cmdenv = parse_key_value_list(self.options.cmdenv,
-                                                   cmdenv_err,
-                                                   self.option_parser.error)
-
-        jobconf_err = 'jobconf argument "%s" is not of the form KEY=VALUE'
-        self.options.jobconf = parse_key_value_list(self.options.jobconf,
-                                                    jobconf_err,
-                                                    self.option_parser.error)
-
-        def parse_commas(cleanup_str):
-            cleanup_error = ('cleanup option %s is not one of '
-                             + ', '.join(CLEANUP_CHOICES))
-            new_cleanup_options = []
-            for choice in cleanup_str.split(','):
-                if choice in CLEANUP_CHOICES:
-                    new_cleanup_options.append(choice)
-                else:
-                    self.option_parser.error(cleanup_error % choice)
-            if ('NONE' in new_cleanup_options and
-                    len(set(new_cleanup_options)) > 1):
-                self.option_parser.error(
-                    'Cannot clean up both nothing and something!')
-            return new_cleanup_options
-
-        if self.options.cleanup is not None:
-            self.options.cleanup = parse_commas(self.options.cleanup)
-        if self.options.cleanup_on_failure is not None:
-            self.options.cleanup_on_failure = parse_commas(
-                self.options.cleanup_on_failure)
+        _fix_custom_options(self.options, self.option_parser)
 
     def job_runner_kwargs(self):
         """Keyword arguments used to create runners when
@@ -474,18 +469,15 @@ class MRJobLauncher(object):
         """
         return {
             'bootstrap_mrjob': self.options.bootstrap_mrjob,
+            'check_input_paths': self.options.check_input_paths,
             'cleanup': self.options.cleanup,
             'cleanup_on_failure': self.options.cleanup_on_failure,
             'cmdenv': self.options.cmdenv,
-            'conf_path': None,
             'conf_paths': self.options.conf_paths,
             'extra_args': self.generate_passthrough_arguments(),
             'file_upload_args': self.generate_file_upload_args(),
-            'hadoop_extra_args': self.options.hadoop_extra_args,
             'hadoop_input_format': self.hadoop_input_format(),
             'hadoop_output_format': self.hadoop_output_format(),
-            'hadoop_streaming_jar': self.options.hadoop_streaming_jar,
-            'hadoop_version': self.options.hadoop_version,
             'input_paths': self.args,
             'interpreter': self.options.interpreter,
             'jobconf': self.jobconf(),
@@ -493,14 +485,16 @@ class MRJobLauncher(object):
             'label': self.options.label,
             'output_dir': self.options.output_dir,
             'owner': self.options.owner,
-            'partitioner': self.partitioner(),
+            'partitioner': self.partitioner() or self.options.partitioner,
             'python_archives': self.options.python_archives,
             'python_bin': self.options.python_bin,
+            'setup': self.options.setup,
             'setup_cmds': self.options.setup_cmds,
             'setup_scripts': self.options.setup_scripts,
             'stdin': self.stdin,
             'steps_interpreter': self.options.steps_interpreter,
             'steps_python_bin': self.options.steps_python_bin,
+            'strict_protocols': self.options.strict_protocols,
             'upload_archives': self.options.upload_archives,
             'upload_files': self.options.upload_files,
         }
@@ -514,7 +508,9 @@ class MRJobLauncher(object):
 
         Re-define this if you want finer control when running jobs locally.
         """
-        return self.job_runner_kwargs()
+        return combine_dicts(
+            self.job_runner_kwargs(),
+            self._get_kwargs_from_opt_group(self.local_opt_group))
 
     def local_job_runner_kwargs(self):
         """Keyword arguments to create create runners when
@@ -525,7 +521,9 @@ class MRJobLauncher(object):
 
         Re-define this if you want finer control when running jobs locally.
         """
-        return self.job_runner_kwargs()
+        return combine_dicts(
+            self.job_runner_kwargs(),
+            self._get_kwargs_from_opt_group(self.local_opt_group))
 
     def emr_job_runner_kwargs(self):
         """Keyword arguments to create create runners when
@@ -538,6 +536,7 @@ class MRJobLauncher(object):
         """
         return combine_dicts(
             self.job_runner_kwargs(),
+            self._get_kwargs_from_opt_group(self.hadoop_emr_opt_group),
             self._get_kwargs_from_opt_group(self.emr_opt_group))
 
     def hadoop_job_runner_kwargs(self):
@@ -551,6 +550,7 @@ class MRJobLauncher(object):
         """
         return combine_dicts(
             self.job_runner_kwargs(),
+            self._get_kwargs_from_opt_group(self.hadoop_emr_opt_group),
             self._get_kwargs_from_opt_group(self.hadoop_opt_group))
 
     ### Default values for Hadoop stuff ###
@@ -582,7 +582,8 @@ class MRJobLauncher(object):
         keyword args we want to set have identical names).
         """
         keys = set(opt.dest for opt in opt_group.option_list)
-        return dict((key, getattr(self.options, key)) for key in keys)
+        return dict((key, getattr(self.options, key)) for key in keys
+                    if key not in _FAKE_OPTIONS)
 
     def generate_passthrough_arguments(self):
         """Returns a list of arguments to pass to subprocesses, either on
@@ -631,7 +632,7 @@ class MRJobLauncher(object):
         """Redirect stdin, stdout, and stderr for automated testing.
 
         You can set stdin, stdout, and stderr to file objects. By
-        default, they'll be set to empty ``StringIO`` objects.
+        default, they'll be set to empty ``BytesIO`` objects.
         You can then access the job's file handles through ``self.stdin``,
         ``self.stdout``, and ``self.stderr``. See :ref:`testing` for more
         information about testing.
@@ -642,11 +643,11 @@ class MRJobLauncher(object):
         ``stdin`` is empty by default. You can set it to anything that yields
         lines::
 
-            mr_job.sandbox(stdin=StringIO('some_data\\n'))
+            mr_job.sandbox(stdin=BytesIO(b'some_data\\n'))
 
         or, equivalently::
 
-            mr_job.sandbox(stdin=['some_data\\n'])
+            mr_job.sandbox(stdin=[b'some_data\\n'])
 
         For convenience, this sandbox() returns self, so you can do::
 
@@ -655,24 +656,28 @@ class MRJobLauncher(object):
         Simple testing example::
 
             mr_job = MRYourJob.sandbox()
-            assert_equal(list(mr_job.reducer('foo', ['bar', 'baz'])), [...])
+            self.assertEqual(list(mr_job.reducer('foo', ['a', 'b'])), [...])
 
         More complex testing example::
 
-            from StringIO import StringIO
+            from BytesIO import BytesIO
+
+            from mrjob.parse import parse_mr_job_stderr
+            from mrjob.protocol import JSONProtocol
 
             mr_job = MRYourJob(args=[...])
 
             fake_input = '"foo"\\t"bar"\\n"foo"\\t"baz"\\n'
-            mr_job.sandbox(stdin=StringIO(fake_input))
+            mr_job.sandbox(stdin=BytesIO(fake_input))
 
             mr_job.run_reducer(link_num=0)
-            assert_equal(mr_job.parse_output(), ...)
-            assert_equal(mr_job.parse_counters(), ...)
+
+            self.assertEqual(mrjob.stdout.getvalue(), ...)
+            self.assertEqual(parse_mr_job_stderr(mr_job.stderr), ...)
         """
-        self.stdin = stdin or StringIO()
-        self.stdout = stdout or StringIO()
-        self.stderr = stderr or StringIO()
+        self.stdin = stdin or BytesIO()
+        self.stdout = stdout or BytesIO()
+        self.stderr = stderr or BytesIO()
 
         return self
 
