@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2009-2013 Yelp and Contributors
+# Copyright 2009-2016 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,8 @@ made public until at least 0.4 if not later or never.
 from optparse import OptionParser
 from optparse import SUPPRESS_USAGE
 
+from mrjob.parse import parse_key_value_list
+from mrjob.parse import parse_port_range_list
 from mrjob.runner import CLEANUP_CHOICES
 
 
@@ -37,18 +39,22 @@ def _append_to_conf_paths(option, opt_str, value, parser):
         parser.values.conf_paths.append(value)
 
 
-def add_protocol_opts(opt_group):
+def _add_protocol_opts(opt_group):
     """Add options related to choosing protocols.
     """
     return [
         opt_group.add_option(
             '--strict-protocols', dest='strict_protocols', default=None,
             action='store_true', help='If something violates an input/output '
-            'protocol then raise an exception'),
+            'protocol then raise an exception (the default)'),
+        opt_group.add_option(
+            '--no-strict-protocols', dest='strict_protocols', default=None,
+            action='store_false', help='If something violates an input/output '
+            'protocol then increment a counter and continue'),
     ]
 
 
-def add_basic_opts(opt_group):
+def _add_basic_opts(opt_group):
     """Options for all command line tools"""
 
     return [
@@ -73,7 +79,7 @@ def add_basic_opts(opt_group):
     ]
 
 
-def add_runner_opts(opt_group, default_runner='local'):
+def _add_runner_opts(opt_group, default_runner='local'):
     """Options for all runners."""
     return [
         opt_group.add_option(
@@ -93,13 +99,13 @@ def add_runner_opts(opt_group, default_runner='local'):
         opt_group.add_option(
             '--cleanup', dest='cleanup', default=None,
             help=('Comma-separated list of which directories to delete when'
-                  ' a job succeeds, e.g. SCRATCH,LOGS. Choices:'
+                  ' a job succeeds, e.g. TMP,LOGS. Choices:'
                   ' %s (default: ALL)' % ', '.join(CLEANUP_CHOICES))),
 
         opt_group.add_option(
             '--cleanup-on-failure', dest='cleanup_on_failure', default=None,
             help=('Comma-separated list of which directories to delete when'
-                  ' a job fails, e.g. SCRATCH,LOGS. Choices:'
+                  ' a job fails, e.g. TMP,LOGS. Choices:'
                   ' %s (default: NONE)' % ', '.join(CLEANUP_CHOICES))),
 
         opt_group.add_option(
@@ -116,7 +122,15 @@ def add_runner_opts(opt_group, default_runner='local'):
 
         opt_group.add_option(
             '--interpreter', dest='interpreter', default=None,
-            help=("Interpreter to run your script, e.g. python or ruby.")),
+            help=('Non-python command to run your script, e.g. "ruby".')),
+
+        # for more info about jobconf:
+        # http://hadoop.apache.org/mapreduce/docs/current/mapred-default.html
+        opt_group.add_option(
+            '--jobconf', dest='jobconf', default=[], action='append',
+            help=('-D arg to pass through to hadoop streaming; should'
+                  ' take the form KEY=VALUE. You can use --jobconf multiple'
+                  ' times.')),
 
         opt_group.add_option(
             '--no-bootstrap-mrjob', dest='bootstrap_mrjob',
@@ -136,6 +150,11 @@ def add_runner_opts(opt_group, default_runner='local'):
             'for EMR, an HDFS path for Hadoop, and a system path for local,' +
             'and must be empty'),
 
+        opt_group.add_option(
+            '--partitioner', dest='partitioner', default=None,
+            help=('Hadoop partitioner class. Deprecated as of v0.5.1 and'
+                  ' will be removed in v0.6.0 (specify in your job instead)')),
+
         opt_group.add_option(
             '--python-archive', dest='python_archives', default=[],
             action='append',
@@ -145,9 +164,9 @@ def add_runner_opts(opt_group, default_runner='local'):
 
         opt_group.add_option(
             '--python-bin', dest='python_bin', default=None,
-            help=("Deprecated. Name/path of alternate python binary for"
-                  " wrapper script and Python mappers/reducers. You can"
-                  " include arguments, e.g. --python-bin 'python -v'")),
+            help=('Alternate python command for Python'
+                  ' mappers/reducers. You can'
+                  ' include arguments, e.g. --python-bin "python -v"')),
 
         opt_group.add_option(
             '-r', '--runner', dest='runner', default=default_runner,
@@ -182,37 +201,27 @@ def add_runner_opts(opt_group, default_runner='local'):
 
         opt_group.add_option(
             '--steps-interpreter', dest='steps_interpreter', default=None,
-            help=("Name/path of alternate interpreter binary to use to query"
-                  " the job about its steps, if different from --interpreter."
-                  " Rarely needed.")),
+            help=("Non-Python command to use to query the job about its"
+                  " steps, if different from --interpreter.")),
 
         opt_group.add_option(
             '--steps-python-bin', dest='steps_python_bin', default=None,
-            help=('Deprecated. Name/path of alternate python binary to use to'
+            help=('Name/path of alternate python command to use to'
                   ' query the job about its steps, if different from the'
-                  ' current Python interpreter. Rarely needed.')),
+                  ' current Python interpreter.')),
     ]
 
 
-def add_hadoop_shared_opts(opt_group):
-    """Options for ``hadoop``, ``local``, and ``emr`` runners"""
+def _add_local_opts(opt_group):
+    """Options for ``inline`` and ``local`` runners."""
     return [
         opt_group.add_option(
             '--hadoop-version', dest='hadoop_version', default=None,
-            help=('Version of Hadoop to specify to EMR or to emulate for -r'
-                  ' local. Default is 0.20.')),
-
-        # for more info about jobconf:
-        # http://hadoop.apache.org/mapreduce/docs/current/mapred-default.html
-        opt_group.add_option(
-            '--jobconf', dest='jobconf', default=[], action='append',
-            help=('-jobconf arg to pass through to hadoop streaming; should'
-                  ' take the form KEY=VALUE. You can use --jobconf multiple'
-                  ' times.')),
+            help=('Specific version of Hadoop to simulate')),
     ]
 
 
-def add_hadoop_emr_opts(opt_group):
+def _add_hadoop_emr_opts(opt_group):
     """Options for ``hadoop`` and ``emr`` runners"""
     return [
         opt_group.add_option(
@@ -227,66 +236,327 @@ def add_hadoop_emr_opts(opt_group):
 
         opt_group.add_option(
             '--label', dest='label', default=None,
-            help='custom prefix for job name, to help us identify the job'),
+            help='alternate label for the job, to help us identify it'),
 
         opt_group.add_option(
             '--owner', dest='owner', default=None,
-            help='custom username to use, to help us identify who ran the'
-            ' job'),
-
-        opt_group.add_option(
-            '--partitioner', dest='partitioner', default=None,
-            help=('Hadoop partitioner class to use to determine how mapper'
-                  ' output should be sorted and distributed to reducers. For'
-                  ' example: org.apache.hadoop.mapred.lib.HashPartitioner')),
-    ]
-
-
-def add_hadoop_opts(opt_group):
-    """Options for ``hadoop`` runner"""
-    return [
-        opt_group.add_option(
-            '--hadoop-bin', dest='hadoop_bin', default=None,
-            help='hadoop binary. Defaults to $HADOOP_HOME/bin/hadoop'),
-
-        opt_group.add_option(
-            '--hdfs-scratch-dir', dest='hdfs_scratch_dir',
-            default=None,
-            help='Scratch space on HDFS (default is tmp/)'),
+            help='user who ran the job (if different from the current user)'),
 
         opt_group.add_option(
             '--check-input-paths', dest='check_input_paths',
-            default=True, action='store_true',
+            default=None, action='store_true',
             help='Check input paths exist before running (the default)'),
 
         opt_group.add_option(
             '--no-check-input-paths', dest='check_input_paths',
-            default=True, action='store_false',
+            default=None, action='store_false',
             help='Skip the checks to ensure all input paths exist'),
     ]
 
 
-def add_emr_opts(opt_group):
+def _add_hadoop_opts(opt_group):
+    """Options for ``hadoop`` runner"""
+    return [
+        opt_group.add_option(
+            '--hadoop-bin', dest='hadoop_bin', default=None,
+            help='path to hadoop binary'),
+
+        opt_group.add_option(
+            '--hadoop-log-dir', dest='hadoop_log_dirs', default=[],
+            action='append', help='Directory to search for'
+            ' hadoop logs in. You can use --hadoop-log-dir multiple times.'),
+
+        opt_group.add_option(
+            '--hadoop-home', dest='hadoop_home',
+            default=None,
+            help='Deprecated hint about where to find hadoop binary and'
+                 ' streaming jar. In most cases mrjob will now find these on'
+                 ' its own. If not, use the --hadoop-bin and'
+                 ' --hadoop-streaming-jar switches.'),
+
+        opt_group.add_option(
+            '--hadoop-tmp-dir', dest='hadoop_tmp_dir',
+            default=None,
+            help='Temp space on HDFS (default is tmp/mrjob)'),
+
+        opt_group.add_option(
+            '--hdfs-scratch-dir', dest='hdfs_scratch_dir',
+            default=None,
+            help='Deprecated alias for --hadoop-tmp-dir'),
+    ]
+
+
+def _add_emr_opts(opt_group):
     """Options for ``emr`` runner"""
+    return (_add_emr_connect_opts(opt_group) +
+            _add_emr_launch_opts(opt_group) +
+            _add_emr_run_opts(opt_group))
+
+
+def _add_emr_connect_opts(opt_group):
+    """Options for connecting to the EMR API."""
+    return [
+        opt_group.add_option(
+            '--aws-region', dest='aws_region', default=None,
+            help=('Region to run EMR jobs in. Default is us-west-2')),
+
+        opt_group.add_option(
+            '--emr-endpoint', dest='emr_endpoint', default=None,
+            help=('Force mrjob to connect to EMR on this endpoint'
+                  ' (e.g. us-west-1.elasticmapreduce.amazonaws.com). Default'
+                  ' is to infer this from aws_region.')),
+
+        opt_group.add_option(
+            '--s3-endpoint', dest='s3_endpoint', default=None,
+            help=("Force mrjob to connect to S3 on this endpoint (e.g."
+                  " s3-us-west-1.amazonaws.com). You usually shouldn't"
+                  " set this; by default mrjob will choose the correct"
+                  " endpoint for each S3 bucket based on its location.")),
+    ]
+
+
+def _add_emr_run_opts(opt_group):
+    """Options for running and monitoring a job on EMR."""
+    return [
+        opt_group.add_option(
+            '--check-emr-status-every', dest='check_emr_status_every',
+            default=None, type='int',
+            help='How often (in seconds) to check status of your EMR job'),
+
+        opt_group.add_option(
+            '--cluster-id', dest='cluster_id', default=None,
+            help='ID of an existing EMR cluster to run our job on'),
+
+        # --ec2-key-pair is used to launch the job, not to monitor it
+        opt_group.add_option(
+            '--ec2-key-pair-file', dest='ec2_key_pair_file', default=None,
+            help='Path to file containing SSH key for EMR'),
+
+        opt_group.add_option(
+            '--emr-action-on-failure', dest='emr_action_on_failure',
+            default=None,
+            help=('Action to take when a step fails'
+                  ' (e.g. TERMINATE_CLUSTER | CANCEL_AND_WAIT | CONTINUE)')),
+
+        opt_group.add_option(
+            '--emr-job-flow-id', dest='emr_job_flow_id', default=None,
+            help='Deprecated alias for --cluster-id'),
+
+        opt_group.add_option(
+            '--hadoop-streaming-jar-on-emr',
+            dest='hadoop_streaming_jar_on_emr', default=None,
+            help=('Local path of the hadoop streaming jar on the EMR node.'
+                  ' Rarely necessary.')),
+
+        opt_group.add_option(
+            '--no-ssh-tunnel', dest='ssh_tunnel',
+            default=None, action='store_false',
+            help=("Don't open an SSH tunnel to the Hadoop job"
+                  " tracker/resource manager")),
+
+        opt_group.add_option(
+            '--pool-wait-minutes', dest='pool_wait_minutes', default=None,
+            type='int',
+            help=('Wait for a number of minutes for a cluster to finish'
+                  ' if a job finishes, run job on its cluster. Otherwise'
+                  " create a new one. (0, the default, means don't wait)")),
+
+        opt_group.add_option(
+            '--ssh-bin', dest='ssh_bin', default=None,
+            help=("Name/path of ssh binary. Arguments are allowed (e.g."
+                  " --ssh-bin 'ssh -v')")),
+
+        opt_group.add_option(
+            '--ssh-bind-ports', dest='ssh_bind_ports', default=None,
+            help=('A list of port ranges that are safe to listen on, delimited'
+                  ' by colons and commas, with syntax like'
+                  ' 2000[:2001][,2003,2005:2008,etc].'
+                  ' Defaults to 40001:40840.')),
+
+        opt_group.add_option(
+            '--ssh-tunnel', dest='ssh_tunnel',
+            default=None, action='store_true',
+            help=('Open an SSH tunnel to the Hadoop job tracker/resource'
+                  ' manager')),
+
+        opt_group.add_option(
+            '--ssh-tunnel-is-closed', dest='ssh_tunnel_is_open',
+            default=None, action='store_false',
+            help='Make ssh tunnel accessible from localhost only'),
+
+        opt_group.add_option(
+            '--ssh-tunnel-is-open', dest='ssh_tunnel_is_open',
+            default=None, action='store_true',
+            help=('Make ssh tunnel accessible from remote hosts (not just'
+                  ' localhost).')),
+
+        opt_group.add_option(
+            '--ssh-tunnel-to-job-tracker', dest='ssh_tunnel_to_job_tracker',
+            default=None, action='store_true',
+            help='Deprecated alias for --ssh-tunnel'),
+    ]
+
+
+def _add_emr_launch_opts(opt_group):
+    """Options for launching a cluster (including bootstrapping)."""
     return [
         opt_group.add_option(
             '--additional-emr-info', dest='additional_emr_info', default=None,
             help='A JSON string for selecting additional features on EMR'),
 
-        opt_group.add_option(
-            '--ami-version', dest='ami_version', default=None,
-            help=(
-                'AMI Version to use (currently 1.0, 2.0, or latest, default'
-                ' latest).')),
-
         opt_group.add_option(
             '--aws-availability-zone', dest='aws_availability_zone',
             default=None,
-            help='Availability zone to run the job flow on'),
+            help='Availability zone to run the cluster on'),
 
         opt_group.add_option(
-            '--aws-region', dest='aws_region', default=None,
-            help='Region to connect to S3 and EMR on (e.g. us-west-1).'),
+            '--ec2-key-pair', dest='ec2_key_pair', default=None,
+            help='Name of the SSH key pair you set up for EMR'),
+
+        opt_group.add_option(
+            '--emr-api-param', dest='emr_api_params',
+            default=[], action='append',
+            help='Additional parameters to pass directly to the EMR API '
+                 ' when creating a cluster. Should take the form KEY=VALUE.'
+                 ' You can use --emr-api-param multiple times.'
+        ),
+
+        opt_group.add_option(
+            '--emr-tag', dest='emr_tags',
+            default=[], action='append',
+            help='Metadata tags to apply to the EMR cluster; '
+                 'should take the form KEY=VALUE. You can use --emr-tag '
+                 'multiple times.'),
+
+        opt_group.add_option(
+            '--iam-endpoint', dest='iam_endpoint', default=None,
+            help=('Force mrjob to connect to IAM on this endpoint'
+                  ' (e.g. iam.us-gov.amazonaws.com)')),
+
+        opt_group.add_option(
+            '--iam-instance-profile', dest='iam_instance_profile',
+            default=None,
+            help=('EC2 instance profile to use for the EMR cluster - see'
+                  ' "Configure IAM Roles for Amazon EMR" in AWS docs')),
+
+        opt_group.add_option(
+            '--iam-service-role', dest='iam_service_role',
+            default=None,
+            help=('IAM service role to use for the EMR cluster -- see'
+                  ' "Configure IAM Roles for Amazon EMR" in AWS docs')),
+
+        opt_group.add_option(
+            '--max-hours-idle', dest='max_hours_idle',
+            default=None, type='float',
+            help=("If we create a persistent cluster, have it automatically"
+                  " terminate itself after it's been idle this many hours.")),
+
+        opt_group.add_option(
+            '--mins-to-end-of-hour', dest='mins_to_end_of_hour',
+            default=None, type='float',
+            help=("If --max-hours-idle is set, control how close to the end"
+                  " of an EC2 billing hour the cluster can automatically"
+                  " terminate itself (default is 5 minutes).")),
+
+        opt_group.add_option(
+            '--no-bootstrap-python', dest='bootstrap_python',
+            action='store_false', default=None,
+            help=("Don't automatically try to install a compatible version"
+                  " of Python at bootstrap time.")),
+
+        opt_group.add_option(
+            '--no-pool-clusters', dest='pool_clusters',
+            action='store_false',
+            help="Don't run our job on a pooled cluster (the default)."),
+
+        opt_group.add_option(
+            '--no-pool-emr-job-flows', dest='pool_emr_job_flows',
+            action='store_false',
+            help="Deprecated alias for --no-pool-clusters"),
+
+        opt_group.add_option(
+            '--pool-clusters', dest='pool_clusters',
+            action='store_true',
+            help='Add to an existing cluster or create a new one that does'
+                 ' not terminate when the job completes. Overrides other '
+                 ' cluster-related options including EC2 instance'
+                 ' configuration. Joins pool "default" if --pool-name is not'
+                 ' specified. WARNING: do not run this without'
+                 ' mrjob terminate-idle-clusters in your crontab;'
+                 ' clusters left idle can quickly become expensive!'),
+
+        opt_group.add_option(
+            '--pool-emr-job-flows', dest='pool_emr_job_flows',
+            action='store_true',
+            help='Deprecated alias for --pool-clusters'),
+
+        opt_group.add_option(
+            '--pool-name', dest='pool_name', action='store',
+            default=None,
+            help=('Specify a pool name to join. Set to "default" if not'
+                  ' specified.')),
+
+        opt_group.add_option(
+            '--s3-log-uri', dest='s3_log_uri', default=None,
+            help='URI on S3 to write logs into'),
+
+        opt_group.add_option(
+            '--s3-scratch-uri', dest='s3_scratch_uri', default=None,
+            help='Deprecated alias for --s3-tmp-dir.'),
+
+        opt_group.add_option(
+            '--s3-sync-wait-time', dest='s3_sync_wait_time', default=None,
+            type='float',
+            help=('How long to wait for S3 to reach eventual consistency. This'
+                  ' is typically less than a second (zero in us-west) but the'
+                  ' default is 5.0 to be safe.')),
+
+        opt_group.add_option(
+            '--s3-tmp-dir', dest='s3_tmp_dir', default=None,
+            help='URI on S3 to use as our temp directory.'),
+
+        opt_group.add_option(
+            '--s3-upload-part-size', dest='s3_upload_part_size', default=None,
+            type='float',
+            help=('Upload files to S3 in parts no bigger than this many'
+                  ' megabytes. Default is 100 MiB. Set to 0 to disable'
+                  ' multipart uploading entirely.')),
+
+        opt_group.add_option(
+            '--no-emr-api-param', dest='no_emr_api_params',
+            default=[], action='append',
+            help='Parameters to be unset when calling EMR API.'
+                 ' You can use --no-emr-api-param multiple times.'
+        ),
+
+        opt_group.add_option(
+            '--visible-to-all-users', dest='visible_to_all_users',
+            default=None, action='store_true',
+            help='Make your cluster is visible to all IAM users on the same'
+                 ' AWS account (the default).'
+        ),
+
+        opt_group.add_option(
+            '--no-visible-to-all-users', dest='visible_to_all_users',
+            default=None, action='store_false',
+            help='Hide your cluster from other IAM users on the same AWS'
+                 ' account.'
+        ),
+
+    ] + _add_emr_bootstrap_opts(opt_group) + _add_emr_instance_opts(opt_group)
+
+
+def _add_emr_bootstrap_opts(opt_group):
+    """Add options having to do with bootstrapping (other than
+    :mrjob-opt:`bootstrap_mrjob`, which is shared with other runners)."""
+    return [
+
+        opt_group.add_option(
+            '--bootstrap', dest='bootstrap', action='append',
+            help=('A shell command to set up libraries etc. before any steps'
+                  ' (e.g. "sudo apt-get -qy install python3"). You may'
+                  ' interpolate files available via URL or locally with Hadoop'
+                  ' Distributed Cache syntax ("sudo dpkg -i foo.deb#")')),
 
         opt_group.add_option(
             '--bootstrap-action', dest='bootstrap_actions', action='append',
@@ -312,6 +582,13 @@ def add_emr_opts(opt_group):
                   ' be made public on S3 due to a limitation of the bootstrap'
                   ' feature. You can use --bootstrap-file more than once.')),
 
+        opt_group.add_option(
+            '--bootstrap-python', dest='bootstrap_python',
+            action='store_true', default=None,
+            help=('Attempt to install a compatible version of Python'
+                  ' at bootstrap time. Currently this only does anything'
+                  ' for Python 3, for which it is enabled by default.')),
+
         opt_group.add_option(
             '--bootstrap-python-package', dest='bootstrap_python_packages',
             action='append', default=[],
@@ -329,31 +606,43 @@ def add_emr_opts(opt_group):
                   ' --bootstrap-script more than once.')),
 
         opt_group.add_option(
-            '--check-emr-status-every', dest='check_emr_status_every',
-            default=None, type='int',
-            help='How often (in seconds) to check status of your EMR job'),
+            '--disable-emr-debugging', dest='enable_emr_debugging',
+            action='store_false',
+            help='Disable storage of Hadoop logs in SimpleDB'),
 
         opt_group.add_option(
-            '--ec2-instance-type', dest='ec2_instance_type', default=None,
-            help=('Type of EC2 instance(s) to launch (e.g. m1.small,'
-                  ' c1.xlarge, m2.xlarge). See'
-                  ' http://aws.amazon.com/ec2/instance-types/ for the full'
-                  ' list.')),
+            '--enable-emr-debugging', dest='enable_emr_debugging',
+            default=None, action='store_true',
+            help='Enable storage of Hadoop logs in SimpleDB'),
+    ]
+
+
+def _add_emr_instance_opts(opt_group):
+    """Add options having to do with instance creation"""
+    return [
+        # AMI
+        opt_group.add_option(
+            '--ami-version', dest='ami_version', default=None,
+            help=('AMI Version to use, e.g. "2.4.11", "3.8.0", "4.0.0"')),
 
         opt_group.add_option(
-            '--ec2-key-pair', dest='ec2_key_pair', default=None,
-            help='Name of the SSH key pair you set up for EMR'),
+            '--release-label', dest='release_label', default=None,
+            help=('Release Label (e.g. "emr-4.0.0"). Overrides'
+                  ' --ami-version')),
 
-        opt_group.add_option(
-            '--ec2-key-pair-file', dest='ec2_key_pair_file', default=None,
-            help='Path to file containing SSH key for EMR'),
-
-        # EMR instance types
+        # instance types
         opt_group.add_option(
             '--ec2-core-instance-type', '--ec2-slave-instance-type',
             dest='ec2_core_instance_type', default=None,
             help='Type of EC2 instance for core (or "slave") nodes only'),
 
+        opt_group.add_option(
+            '--ec2-instance-type', dest='ec2_instance_type', default=None,
+            help=('Type of EC2 instance(s) to launch (e.g. m1.medium,'
+                  ' c3.xlarge, r3.xlarge). See'
+                  ' http://aws.amazon.com/ec2/instance-types/ for the full'
+                  ' list.')),
+
         opt_group.add_option(
             '--ec2-master-instance-type', dest='ec2_master_instance_type',
             default=None,
@@ -364,77 +653,7 @@ def add_emr_opts(opt_group):
             default=None,
             help='Type of EC2 instance for task nodes only'),
 
-        # EMR instance bid prices
-        opt_group.add_option(
-            '--ec2-core-instance-bid-price',
-            dest='ec2_core_instance_bid_price', default=None,
-            help=(
-                'Bid price to specify for core (or "slave") nodes when'
-                ' setting them up as EC2 spot instances (you probably only'
-                ' want to set a bid price for task instances).')
-            ),
-
-        opt_group.add_option(
-            '--ec2-master-instance-bid-price',
-            dest='ec2_master_instance_bid_price', default=None,
-            help=(
-                'Bid price to specify for the master node when setting it up '
-                'as an EC2 spot instance (you probably only want to set '
-                'a bid price for task instances).')
-            ),
-
-        opt_group.add_option(
-            '--ec2-task-instance-bid-price',
-            dest='ec2_task_instance_bid_price', default=None,
-            help=(
-                'Bid price to specify for task nodes when '
-                'setting them up as EC2 spot instances.')
-            ),
-
-        opt_group.add_option(
-            '--emr-endpoint', dest='emr_endpoint', default=None,
-            help=('Optional host to connect to when communicating with S3'
-                  ' (e.g. us-west-1.elasticmapreduce.amazonaws.com). Default'
-                  ' is to infer this from aws_region.')),
-
-        opt_group.add_option(
-            '--emr-job-flow-id', dest='emr_job_flow_id', default=None,
-            help='ID of an existing EMR job flow to use'),
-
-        opt_group.add_option(
-            '--enable-emr-debugging', dest='enable_emr_debugging',
-            default=None, action='store_true',
-            help='Enable storage of Hadoop logs in SimpleDB'),
-
-        opt_group.add_option(
-            '--disable-emr-debugging', dest='enable_emr_debugging',
-            action='store_false',
-            help='Disable storage of Hadoop logs in SimpleDB'),
-
-        opt_group.add_option(
-            '--hadoop-streaming-jar-on-emr',
-            dest='hadoop_streaming_jar_on_emr', default=None,
-            help=('Local path of the hadoop streaming jar on the EMR node.'
-                  ' Rarely necessary.')),
-
-        opt_group.add_option(
-            '--max-hours-idle', dest='max_hours_idle',
-            default=None, type='float',
-            help=("If we create a persistent job flow, have it automatically"
-                  " terminate itself after it's been idle this many hours.")),
-
-        opt_group.add_option(
-            '--mins-to-end-of-hour', dest='mins_to_end_of_hour',
-            default=None, type='float',
-            help=("If --max-hours-idle is set, control how close to the end"
-                  " of an EC2 billing hour the job flow can automatically"
-                  " terminate itself (default is 5 minutes).")),
-
-        opt_group.add_option(
-            '--no-pool-emr-job-flows', dest='pool_emr_job_flows',
-            action='store_false',
-            help="Don't try to run our job on a pooled job flow."),
-
+        # instance number
         opt_group.add_option(
             '--num-ec2-instances', dest='num_ec2_instances', default=None,
             type='int',
@@ -456,91 +675,102 @@ def add_emr_opts(opt_group):
             help=('Number of EC2 instances to start as task '
                   'nodes. Incompatible with --num-ec2-instances.')),
 
+        # bid price
         opt_group.add_option(
-            '--pool-emr-job-flows', dest='pool_emr_job_flows',
-            action='store_true',
-            help='Add to an existing job flow or create a new one that does'
-                 ' not terminate when the job completes. Overrides other job'
-                 ' flow-related options including EC2 instance configuration.'
-                 ' Joins pool "default" if emr_job_flow_pool_name is not'
-                 ' specified. WARNING: do not run this without'
-                 ' mrjob.tools.emr.terminate_idle_job_flows in your crontab;'
-                 ' job flows left idle can quickly become expensive!'),
+            '--ec2-core-instance-bid-price',
+            dest='ec2_core_instance_bid_price', default=None,
+            help=(
+                'Bid price to specify for core (or "slave") nodes when'
+                ' setting them up as EC2 spot instances (you probably only'
+                ' want to set a bid price for task instances).')
+        ),
 
         opt_group.add_option(
-            '--pool-name', dest='emr_job_flow_pool_name', action='store',
-            default=None,
-            help=('Specify a pool name to join. Set to "default" if not'
-                  ' specified.')),
+            '--ec2-master-instance-bid-price',
+            dest='ec2_master_instance_bid_price', default=None,
+            help=(
+                'Bid price to specify for the master node when setting it up '
+                'as an EC2 spot instance (you probably only want to set '
+                'a bid price for task instances).')
+        ),
 
         opt_group.add_option(
-            '--pool-wait-minutes', dest='pool_wait_minutes', default=0,
-            type='int',
-            help=('Wait for a number of minutes for a job flow to finish'
-                  ' if a job finishes, pick up their job flow. Otherwise'
-                  ' create a new one. (default 0)')),
-
-        opt_group.add_option(
-            '--s3-endpoint', dest='s3_endpoint', default=None,
-            help=('Host to connect to when communicating with S3 (e.g.'
-                  ' s3-us-west-1.amazonaws.com). Default is to infer this from'
-                  ' region (see --aws-region).')),
-
-        opt_group.add_option(
-            '--s3-log-uri', dest='s3_log_uri', default=None,
-            help='URI on S3 to write logs into'),
-
-        opt_group.add_option(
-            '--s3-scratch-uri', dest='s3_scratch_uri', default=None,
-            help='URI on S3 to use as our temp directory.'),
-
-        opt_group.add_option(
-            '--s3-sync-wait-time', dest='s3_sync_wait_time', default=None,
-            type='float',
-            help=('How long to wait for S3 to reach eventual consistency. This'
-                  ' is typically less than a second (zero in us-west) but the'
-                  ' default is 5.0 to be safe.')),
-
-        opt_group.add_option(
-            '--ssh-bin', dest='ssh_bin', default=None,
-            help=("Name/path of ssh binary. Arguments are allowed (e.g."
-                  " --ssh-bin 'ssh -v')")),
-
-        opt_group.add_option(
-            '--ssh-bind-ports', dest='ssh_bind_ports', default=None,
-            help=('A list of port ranges that are safe to listen on, delimited'
-                  ' by colons and commas, with syntax like'
-                  ' 2000[:2001][,2003,2005:2008,etc].'
-                  ' Defaults to 40001:40840.')),
-
-        opt_group.add_option(
-            '--ssh-tunnel-is-closed', dest='ssh_tunnel_is_open',
-            default=None, action='store_false',
-            help='Make ssh tunnel accessible from localhost only'),
-
-        opt_group.add_option(
-            '--ssh-tunnel-is-open', dest='ssh_tunnel_is_open',
-            default=None, action='store_true',
-            help=('Make ssh tunnel accessible from remote hosts (not just'
-                  ' localhost).')),
-
-        opt_group.add_option(
-            '--ssh-tunnel-to-job-tracker', dest='ssh_tunnel_to_job_tracker',
-            default=None, action='store_true',
-            help='Open up an SSH tunnel to the Hadoop job tracker'),
-        opt_group.add_option(
-            '--visible-to-all-users', dest='visible_to_all_users',
-            default=None, action='store_true',
-            help='Whether the job flow is visible to all IAM users of the AWS'
-                 ' account associated with the job flow. If this value is set'
-                 ' to True, all IAM users of that AWS account can view and'
-                 ' (if they have the proper policy permissions set) manage'
-                 ' the job flow. If it is set to False, only the IAM user'
-                 ' that created the job flow can view and manage it.'),
+            '--ec2-task-instance-bid-price',
+            dest='ec2_task_instance_bid_price', default=None,
+            help=(
+                'Bid price to specify for task nodes when '
+                'setting them up as EC2 spot instances.')
+        ),
     ]
 
 
-def print_help_for_groups(*args):
+def _print_help_for_groups(*args):
     option_parser = OptionParser(usage=SUPPRESS_USAGE, add_help_option=False)
     option_parser.option_groups = args
     option_parser.print_help()
+
+
+def _alphabetize_options(opt_group):
+    opt_group.option_list.sort(key=lambda opt: opt.dest or '')
+
+
+def _fix_custom_options(options, option_parser):
+    """Update *options* to handle KEY=VALUE options, etc."""
+    if hasattr(options, 'cmdenv'):
+        cmdenv_err = '--cmdenv argument %r is not of the form KEY=VALUE'
+        options.cmdenv = parse_key_value_list(options.cmdenv,
+                                              cmdenv_err,
+                                              option_parser.error)
+
+    def parse_commas(cleanup_str):
+        cleanup_error = ('cleanup option %s is not one of ' +
+                         ', '.join(CLEANUP_CHOICES))
+        new_cleanup_options = []
+        for choice in cleanup_str.split(','):
+            if choice in CLEANUP_CHOICES:
+                new_cleanup_options.append(choice)
+            else:
+                option_parser.error(cleanup_error % choice)
+        if ('NONE' in new_cleanup_options and
+                len(set(new_cleanup_options)) > 1):
+            option_parser.error(
+                'Cannot clean up both nothing and something!')
+
+        return new_cleanup_options
+
+    if getattr(options, 'cleanup', None):
+        options.cleanup = parse_commas(options.cleanup)
+
+    if getattr(options, 'cleanup_on_failure', None):
+        options.cleanup_on_failure = parse_commas(options.cleanup_on_failure)
+
+    if hasattr(options, 'emr_api_params'):
+        emr_api_err = (
+            '--emr-api-params argument %r is not of the form KEY=VALUE')
+        options.emr_api_params = parse_key_value_list(options.emr_api_params,
+                                                      emr_api_err,
+                                                      option_parser.error)
+
+        if hasattr(options, 'no_emr_api_params'):
+                for param in options.no_emr_api_params:
+                    options.emr_api_params[param] = None
+
+    if hasattr(options, 'emr_tags'):
+        emr_tag_err = '--emr-tag argument %r is not of the form KEY=VALUE'
+        options.emr_tags = parse_key_value_list(options.emr_tags,
+                                                emr_tag_err,
+                                                option_parser.error)
+
+    if hasattr(options, 'jobconf'):
+        jobconf_err = '--jobconf argument %r is not of the form KEY=VALUE'
+        options.jobconf = parse_key_value_list(options.jobconf,
+                                               jobconf_err,
+                                               option_parser.error)
+
+    if getattr(options, 'ssh_bind_ports', None):
+        try:
+            ports = parse_port_range_list(options.ssh_bind_ports)
+        except ValueError as e:
+            option_parser.error('invalid port range list %r: \n%s' %
+                                (options.ssh_bind_ports, e.args[0]))
+            options.ssh_bind_ports = ports
